import "./DateRangeInput.css";
import { Box } from "../../box";
import { useCallback, useImperativeHandle, useRef, useState } from "react";
import { ClearInputGroupContext, InputGroup, useInputGroupProps } from "../../input-group";
import { CrossButton, IconButton } from "../../button";
import { DateInputMask, useDateInput } from "./useDateInput";
import { DisclosureArrow } from "../../disclosure";
import { Divider } from "../../divider";
import { Keys, augmentElement, cssModule, forwardRef, isNil, isNilOrEmpty, isNumber, mergeProps, omitProps, useAutoFocus, useControllableState, useEventCallback, useFocusWithin, useMergedRefs } from "../../shared";
import { Item } from "../../collection";
import { Menu, MenuTrigger } from "../../menu";
import { useFieldInputProps } from "../../field";
import { useToolbarProps } from "../../toolbar";
import { jsx as _jsx } from "react/jsx-runtime";
import { jsxs as _jsxs } from "react/jsx-runtime";
const DateInput = forwardRef(({
  value,
  placeholder = "dd/mm/yyyy",
  required,
  validationState,
  min,
  max,
  onChange,
  onDateChange,
  autoFocus,
  disabled,
  readOnly,
  ...rest
}, ref) => {
  const inputRef = useMergedRefs(ref);
  useAutoFocus(inputRef, {
    isDisabled: !autoFocus || disabled || readOnly,
    delay: isNumber(autoFocus) ? autoFocus : undefined
  });
  const dateProps = useDateInput({
    value,
    min,
    max,
    onChange,
    onDateChange,
    forwardedRef: inputRef
  });
  return /*#__PURE__*/_jsx("input", { ...mergeProps(rest, {
      placeholder,
      className: "o-ui-date-range-input-date-input",
      type: "text",
      disabled,
      readOnly,
      "aria-required": required ? true : undefined,
      "aria-invalid": validationState === "invalid" ? true : undefined,
      ref: inputRef
    }, dateProps)
  });
});
const RangeInput = forwardRef((props, ref) => {
  // eslint-disable-next-line @typescript-eslint/no-unused-vars
  const [_, isInField] = useFieldInputProps();
  const [inputGroupProps, isInGroup] = useInputGroupProps();
  const {
    startDate,
    endDate,
    placeholder,
    min,
    max,
    required,
    validationState,
    onDatesChange,
    onFocus,
    onBlur,
    autoFocus,
    fluid,
    disabled,
    readOnly,
    active,
    focus = false,
    hover,
    name,
    ...rest
  } = mergeProps(props, inputGroupProps);
  const [hasFocus, setHasFocus] = useState(focus);
  const containerRef = useRef();
  const startDateRef = useRef();
  const endDateRef = useRef();
  useImperativeHandle(ref, () => {
    const element = containerRef.current;

    element.focus = () => {
      var _startDateRef$current;

      (_startDateRef$current = startDateRef.current) === null || _startDateRef$current === void 0 ? void 0 : _startDateRef$current.focus();
    };

    return element;
  });
  const handleStartDateChange = useEventCallback((event, newDate) => {
    if (!isNil(newDate) && !isNil(endDate) && newDate > endDate) {
      newDate = endDate;
    } // Defering because useDateInput is used in controlled mode and otherwise the value will not be updated when the value is clamped.
    // requestAnimationFrame(() => {
    //     if (!isNil(newDate)) {
    //         endDateRef.current?.focus();
    //     }
    // });


    if (!isNil(onDatesChange)) {
      onDatesChange(event, newDate, endDate);
    }
  });
  const handleEndDateChange = useEventCallback((event, newDate) => {
    if (!isNil(newDate) && !isNil(startDate) && newDate < startDate) {
      newDate = startDate;
    }

    if (!isNil(onDatesChange)) {
      onDatesChange(event, startDate, newDate);
    }
  });
  const handleStartDateInputValueChange = useEventCallback(event => {
    if (event.target.value.length === DateInputMask.length) {
      // Defering because useDateInput is used in controlled mode and otherwise the value will not be updated when the value is clamped.
      requestAnimationFrame(() => {
        var _endDateRef$current;

        (_endDateRef$current = endDateRef.current) === null || _endDateRef$current === void 0 ? void 0 : _endDateRef$current.focus();
      });
    } // // @ts-ignore
    // const newCharacter = event.nativeEvent.data;
    // console.log(event.target.value);
    // if (/\d/.test(newCharacter)) {
    //     console.log(event.target.value);
    // }

  });
  const handleEndDateInputValueChange = useEventCallback(event => {
    // @ts-ignore
    const newCharacter = event.nativeEvent.data; // If the new character is not a digit, we don't want to do anything since the new character will be removed by the mask input.
    // The digit is test with a regex because this is how our mask input third party is doing it and we want to be consistant.

    if (/\d/.test(newCharacter)) {
      if (isNilOrEmpty(event.target.value)) {
        var _startDateRef$current2;

        (_startDateRef$current2 = startDateRef.current) === null || _startDateRef$current2 === void 0 ? void 0 : _startDateRef$current2.focus();
      }
    }
  });
  const handleClearDates = useEventCallback(event => {
    startDateRef === null || startDateRef === void 0 ? void 0 : startDateRef.current.focus();

    if (!isNil(onDatesChange)) {
      onDatesChange(event, null, null);
    }
  });
  const handleContainerKeyDown = useEventCallback(event => {
    if (event.key === Keys.esc) {
      event.preventDefault();
      handleClearDates(event);
    }
  });
  const handleEndDateKeyDown = useEventCallback(event => {
    if (event.key === Keys.backspace) {
      var _endDateRef$current2;

      if (isNilOrEmpty((_endDateRef$current2 = endDateRef.current) === null || _endDateRef$current2 === void 0 ? void 0 : _endDateRef$current2.value)) {
        var _startDateRef$current3;

        (_startDateRef$current3 = startDateRef.current) === null || _startDateRef$current3 === void 0 ? void 0 : _startDateRef$current3.focus();
      }
    }
  });
  const focusWithinProps = useFocusWithin({
    onFocus: useEventCallback(event => {
      setHasFocus(true);

      if (!isNil(onFocus)) {
        onFocus(event);
      }
    }),
    onBlur: useEventCallback(event => {
      setHasFocus(false);

      if (!isNil(onBlur)) {
        onBlur(event);
      }
    })
  });
  const hasValue = !isNil(startDate) || !isNil(endDate);
  return /*#__PURE__*/_jsxs(Box, { ...mergeProps(rest, {
      onKeyDown: handleContainerKeyDown,
      className: cssModule("o-ui-date-range-input", validationState, fluid && "fluid", disabled && "disabled", readOnly && "readonly", active && "active", hasFocus && "focus", hover && "hover", isInGroup && "in-group"),
      role: !isInField ? "group" : undefined,
      ref: containerRef
    }, focusWithinProps),
    children: [/*#__PURE__*/_jsx(DateInput, {
      value: startDate,
      placeholder: placeholder,
      required: required,
      validationState: validationState,
      min: min,
      max: max,
      onChange: handleStartDateInputValueChange,
      onDateChange: handleStartDateChange,
      autoFocus: autoFocus,
      disabled: disabled,
      readOnly: readOnly,
      name: !isNil(name) ? `${name}-start-date` : undefined,
      ref: startDateRef
    }), /*#__PURE__*/_jsx(Divider, {
      orientation: "vertical",
      className: "o-ui-date-range-input-divider"
    }), /*#__PURE__*/_jsx(DateInput, {
      value: endDate,
      placeholder: placeholder,
      required: required,
      validationState: validationState,
      min: min,
      max: max,
      onChange: handleEndDateInputValueChange,
      onDateChange: handleEndDateChange,
      onKeyDown: handleEndDateKeyDown,
      disabled: disabled,
      readOnly: readOnly,
      name: !isNil(name) ? `${name}-end-date` : undefined,
      tabIndex: hasFocus ? 0 : -1,
      ref: endDateRef
    }), hasValue && !disabled && !readOnly && /*#__PURE__*/_jsx(ClearInputGroupContext, {
      children: /*#__PURE__*/_jsx(CrossButton, {
        onClick: handleClearDates,
        size: "xs",
        condensed: true,
        className: "o-ui-date-range-input-clear-button",
        "aria-label": "Clear dates"
      })
    })]
  });
});
export function InnerDateRangeInput(props) {
  const [toolbarProps] = useToolbarProps();
  const [fieldProps] = useFieldInputProps();
  const [inputGroupProps, isInGroup] = useInputGroupProps();
  const {
    startDate: startDateProp,
    endDate: endDateProp,
    defaultStartDate,
    defaultEndDate,
    placeholder,
    min,
    max,
    required,
    validationState,
    onDatesChange,
    onFocus,
    onBlur,
    presets,
    autoFocus,
    fluid,
    disabled,
    readOnly,
    active,
    focus = false,
    hover,
    name,
    as = "div",
    forwardedRef,
    ...rest
  } = mergeProps(props, omitProps(toolbarProps, ["orientation"]), fieldProps, inputGroupProps);
  const [startDate, setStartDate] = useControllableState(startDateProp, defaultStartDate, null);
  const [endDate, setEndDate] = useControllableState(endDateProp, defaultEndDate, null);
  const containerRef = useRef();
  const rangeRef = useRef();
  const presetButtonRef = useRef();
  useImperativeHandle(forwardedRef, () => {
    // For presets, used the input group container as the ref element.
    if (!isNil(presets)) {
      const element = containerRef.current;

      element.focus = () => {
        var _rangeRef$current;

        (_rangeRef$current = rangeRef.current) === null || _rangeRef$current === void 0 ? void 0 : _rangeRef$current.focus();
      };

      return element;
    }

    return rangeRef.current;
  });
  const applyDates = useCallback((event, newStartDate, newEndDate) => {
    if (startDate !== newStartDate || endDate !== newEndDate) {
      setStartDate(newStartDate);
      setEndDate(newEndDate);

      if (!isNil(onDatesChange)) {
        onDatesChange(event, newStartDate, newEndDate);
      }
    }
  }, [onDatesChange, startDate, setStartDate, endDate, setEndDate]);
  const handleSelectPreset = useEventCallback((event, keys) => {
    const index = parseInt(keys[0]);
    const preset = presets[index];

    if (!isNil(preset)) {
      var _presetButtonRef$curr;

      applyDates(event, preset.startDate, preset.endDate);
      (_presetButtonRef$curr = presetButtonRef.current) === null || _presetButtonRef$curr === void 0 ? void 0 : _presetButtonRef$curr.focus();
    }
  });

  const rangeMarkup = /*#__PURE__*/_jsx(RangeInput, {
    startDate: startDate,
    endDate: endDate,
    placeholder: placeholder,
    min: min,
    max: max,
    required: required,
    validationState: validationState,
    onDatesChange: applyDates,
    onFocus: onFocus,
    onBlur: onBlur,
    autoFocus: autoFocus,
    fluid: fluid,
    disabled: disabled,
    readOnly: readOnly,
    active: active,
    focus: focus,
    hover: hover,
    name: name,
    ref: rangeRef
  });

  if (!isNil(presets)) {
    return /*#__PURE__*/_jsxs(InputGroup, { ...mergeProps(rest, {
        as,
        ref: containerRef
      }),
      children: [rangeMarkup, /*#__PURE__*/_jsxs(MenuTrigger, {
        children: [/*#__PURE__*/_jsx(IconButton, {
          disabled: disabled || readOnly,
          "aria-label": "Date presets",
          ref: presetButtonRef,
          children: /*#__PURE__*/_jsx(DisclosureArrow, {})
        }), /*#__PURE__*/_jsx(Menu, {
          onSelectionChange: handleSelectPreset,
          children: presets.map((x, index) =>
          /*#__PURE__*/
          // eslint-disable-next-line react/no-array-index-key
          _jsx(Item, {
            children: x.text
          }, index.toString()))
        })]
      })]
    });
  }

  return /*#__PURE__*/_jsx(ClearInputGroupContext, {
    children: augmentElement(rangeMarkup, mergeProps(rest, {
      className: isInGroup ? "o-ui-date-range-input-in-group" : undefined,
      as,
      ref: containerRef
    }))
  });
}
export const DateRangeInput = forwardRef((props, ref) => /*#__PURE__*/_jsx(InnerDateRangeInput, { ...props,
  forwardedRef: ref
}));
DateRangeInput.displayName = "DateRangeInput";