import { isNil, isNumber, mergeProps, useAutoFocusChild, useControllableState, useEventCallback, useFocusManager, useFocusScope, useId, useMergedRefs } from "../../shared";
import { useCallback } from "react";
import { useOverlayPosition } from "./useOverlayPosition";
import { useOverlayTrigger } from "./useOverlayTrigger";
import { isTargetParent } from "./isTargetParent";
import { usePopupLightDismiss } from "./usePopupLightDismiss";
import { useRestoreFocus } from "./useRestoreFocus";
export function usePopup(type, {
  id,
  open,
  defaultOpen,
  onOpenChange,
  hideOnEscape = true,
  hideOnLeave = true,
  hideOnOutsideClick,
  restoreFocus = true,
  autoFocus,
  autoFocusOptions = {},
  trigger = "click",
  hasArrow = false,
  position,
  offset,
  disabled,
  allowFlip = true,
  allowPreventOverflow = true,
  boundaryElement,
  keyProp
} = {}) {
  const [isOpen, setIsOpen] = useControllableState(open, defaultOpen, false);
  const [focusScope, setFocusRef] = useFocusScope();
  const updateIsOpen = useCallback((event, newValue) => {
    if (isOpen !== newValue) {
      setIsOpen(newValue);

      if (!isNil(onOpenChange)) {
        onOpenChange(event, newValue);
      }
    }
  }, [onOpenChange, isOpen, setIsOpen]);
  const triggerProps = useOverlayTrigger(isOpen, {
    trigger,
    onShow: useEventCallback(event => {
      updateIsOpen(event, true);
    }),
    onHide: useEventCallback(event => {
      // Prevent from closing when the focus goes to an element of the overlay on opening.
      if (!isTargetParent(event.relatedTarget, overlayRef)) {
        updateIsOpen(event, false);
      }
    }),
    hideOnLeave,
    isDisabled: disabled
  });
  const {
    triggerRef,
    overlayRef: overlayPositionRef,
    arrowRef
  } = useOverlayPosition({
    position,
    offset,
    allowFlip,
    allowPreventOverflow,
    boundaryElement,
    hasArrow
  });
  const overlayRef = useMergedRefs(overlayPositionRef, setFocusRef);
  const overlayDismissProps = usePopupLightDismiss(triggerRef, overlayRef, {
    trigger,
    onHide: useEventCallback(event => {
      updateIsOpen(event, false);
    }),
    hideOnEscape,
    hideOnLeave,
    hideOnOutsideClick
  });
  const restoreFocusProps = useRestoreFocus(focusScope, {
    isDisabled: !restoreFocus || !isOpen
  });
  const focusManager = useFocusManager(focusScope, {
    keyProp
  });
  useAutoFocusChild(focusManager, { ...autoFocusOptions,
    isDisabled: !autoFocus || !isOpen,
    delay: isNumber(autoFocus) ? autoFocus : undefined,
    onNotFound: useEventCallback(() => {
      var _overlayRef$current;

      (_overlayRef$current = overlayRef.current) === null || _overlayRef$current === void 0 ? void 0 : _overlayRef$current.focus();
    })
  });
  const overlayId = useId(id, "o-ui-overlay");
  return {
    isOpen,
    setIsOpen: updateIsOpen,
    focusScope,
    focusManager,
    triggerProps: mergeProps({
      tabIndex: !restoreFocus && isOpen ? -1 : undefined,
      "aria-haspopup": type,
      "aria-expanded": isOpen,
      "aria-controls": isOpen ? overlayId : undefined,
      ref: triggerRef
    }, triggerProps),
    overlayProps: mergeProps({
      id: overlayId,
      show: isOpen,
      tabIndex: -1,
      ref: overlayRef
    }, overlayDismissProps, restoreFocusProps),
    arrowProps: !hasArrow ? {} : {
      ref: arrowRef
    }
  };
}