import "./Tooltip.css";
import { Children, useCallback } from "react";
import { Overlay, OverlayArrow, isTargetParent, useOverlayLightDismiss, useOverlayPosition, useOverlayTrigger } from "../../overlay";
import { TooltipTriggerContext } from "./TooltipTriggerContext";
import { augmentElement, forwardRef, isNil, mergeProps, resolveChildren, useControllableState, useEventCallback, useId, useMergedRefs } from "../../shared";
import { jsx as _jsx } from "react/jsx-runtime";
import { jsxs as _jsxs } from "react/jsx-runtime";
export function parseTooltipTrigger(children) {
  const array = Children.toArray(resolveChildren(children));

  if (array.length !== 2) {
    throw new Error("A tooltip trigger must have exactly 2 children.");
  }

  return array;
}
export function InnerTooltipTrigger({
  open,
  defaultOpen,
  position: positionProp = "top",
  onOpenChange,
  disabled,
  allowFlip = true,
  allowPreventOverflow = true,
  containerElement,
  zIndex = 10000,
  as = "div",
  children,
  forwardedRef,
  ...rest
}) {
  const [isOpen, setIsOpen] = useControllableState(open, defaultOpen, false);
  const updateIsOpen = useCallback((event, newValue) => {
    if (isOpen !== newValue) {
      setIsOpen(newValue);

      if (!isNil(onOpenChange)) {
        onOpenChange(event, newValue);
      }
    }
  }, [onOpenChange, isOpen, setIsOpen]);
  const {
    triggerRef,
    overlayRef: overlayPositionRef,
    arrowRef
  } = useOverlayPosition({
    hasArrow: true,
    position: positionProp,
    allowFlip,
    allowPreventOverflow,
    boundaryElement: containerElement
  });
  const overlayRef = useMergedRefs(overlayPositionRef, forwardedRef);
  const triggerProps = useOverlayTrigger(isOpen, {
    trigger: "hover",
    onShow: useEventCallback(event => {
      updateIsOpen(event, true);
    }),
    onHide: useEventCallback(event => {
      // Prevent from closing when the focus goes to an element of the overlay on opening.
      if (!isTargetParent(event.relatedTarget, overlayRef)) {
        updateIsOpen(event, false);
      }
    }),
    hideOnLeave: true,
    isDisabled: disabled
  });
  const overlayDismissProps = useOverlayLightDismiss(overlayRef, {
    trigger: "hover",
    onHide: useEventCallback(event => {
      // Ignore events related to the trigger.
      if (!isTargetParent(event.target, triggerRef) && event.relatedTarget !== triggerRef.current) {
        updateIsOpen(event, false);
      }
    }),
    hideOnEscape: true,
    hideOnLeave: true,
    hideOnOutsideClick: false
  });
  const [trigger, tooltip] = parseTooltipTrigger(children);
  const tooltipId = useId(tooltip.props.id, "o-ui-tooltip");
  const triggerWithDescribedBy = augmentElement(trigger, {
    "aria-describedby": isOpen ? tooltipId : undefined
  }); // HACK: a disabled element doesn't fire event, therefore the element is wrapped in a div.

  const triggerElement = !triggerWithDescribedBy.props.disabled ? triggerWithDescribedBy : /*#__PURE__*/_jsx("div", {
    className: "o-ui-tooltip-disabled-wrapper",
    children: triggerWithDescribedBy
  });
  const triggerMarkup = augmentElement(triggerElement, mergeProps(triggerProps, {
    ref: triggerRef
  }));
  const tooltipMarkup = augmentElement(tooltip, {
    id: tooltipId
  });
  return /*#__PURE__*/_jsxs(TooltipTriggerContext.Provider, {
    value: {
      isOpen
    },
    children: [triggerMarkup, /*#__PURE__*/_jsxs(Overlay, { ...mergeProps(rest, {
        show: isOpen,
        borderOffset: "var(--o-ui-global-scale-charlie)",
        zIndex,
        as,
        ref: overlayRef
      }, overlayDismissProps),
      children: [tooltipMarkup, /*#__PURE__*/_jsx(OverlayArrow, {
        ref: arrowRef
      })]
    })]
  });
}
export const TooltipTrigger = forwardRef((props, ref) => /*#__PURE__*/_jsx(InnerTooltipTrigger, { ...props,
  forwardedRef: ref
}));
TooltipTrigger.displayName = "TooltipTrigger";